<?php
/*
 * lt_exercise.php - Vips plugin for Stud.IP
 * Copyright (c) 2003-2005  Erik Schmitt, Philipp Hgelmeyer
 * Copyright (c) 2005-2006  Christa Deiwiks
 * Copyright (c) 2006-2008  Elmar Ludwig, Martin Schrder
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 */

require_once 'exercises/exercise.php';

/**
 * XXX detailed description
 *
 * @author    XXX
 * @copyright XXX
 * @abstract
 */
class lt_exercise extends Exercise
{
    var $answerDistance;  // 0: none, 1: Levenshtein, 2: Soundex

    // constructor
    function lt_exercise($xml = null)
    {
        parent::Exercise($xml);

        $this->type = 'lt_exercise';

        if (!isset($xml)) {  // construct exercise using request parameters
            $this->answerArray    = $this->answerArrayFromInput();
            $this->correctArray   = $this->correctArrayFromInput();
            $this->numberOfAnswers = count($this->answerArray);
            $this->answerDistance = vips_get_int_param('answer_distance');
        }
        else {  // construct exercise using XML
            $dom_xml = $this->getDomXML($xml);
            $answers = $dom_xml->get_elements_by_tagname('Answer');

            foreach ($answers as $node)
            {
                $this->answerArray[] = utf8_decode($node->get_content());
                $this->correctArray[] = $node->get_attribute('Correct');
            }

            $this->numberOfAnswers = count($this->answerArray);
            $this->answerDistance  = reset($this->getTagsFromXML($xml, 'AnswerDistance'));
        }
    }



    /**
     * Counts the exercise's number of answers.
     *
     * @returns The number of answers in the exercise
     */
    function getNumberOfAnswers()
    {
        return $this->numberOfAnswers;
    }



    /**
     * Returns all variables which are peculiar for this exercise type.
     *
     * @returns An array with the key 'answer_distance'
     */
    function getQuirks()
    {
        return array(
            'answer_distance' => $this->answerDistance);
    }



    function setDefaultValues()
    {
        $this->numberOfAnswers = 5;
    }



    /**
     *    Evaluates a solution.
     *
     *    @abstract
     *    @access public
     *    @returns int (from 0 to 100 ) as an % value
     */

    function evaluate ($exercise,$solution)
    {
        $dom_xml = $this->getDomXML($exercise);
        $answers = $dom_xml->get_elements_by_tagname("Answer");

        $answer_array = array();
        $correct_array = array();

        foreach ($answers as $node) {
            $answer_array[]  = utf8_decode($node->get_content());
            $correct_array[] = $node->get_attribute("Correct");
        }

        $answer_distance = $this->getTagsFromXML($exercise, "AnswerDistance");
        $answer_distance = $answer_distance[0];

        $student_array = $this->getTagsFromXML($solution, "answer");
        $studentSolution = $student_array[0];

        $similarity = 0;
        $safe = false;
        $studentSolution = normalizeText($studentSolution);

        if ($studentSolution == '')
        {
            $m_return[1] = 0;       // no points
            $m_return[2] = 1;
            $m_return['percent'] = 0;
            $m_return['safe']    = 1;
            return $m_return;
        }

        for ($i = 0; $i < count($answer_array); ++$i)
        {
            $musterLoesung = $answer_array[$i];
            $musterLoesung = normalizeText($musterLoesung);
            $divisor = max(strlen($musterLoesung), strlen($studentSolution));
            $similarity_temp = 0;

            if ($musterLoesung == $studentSolution)
            {
                $similarity_temp = 1;
            }
            else if ($answer_distance == 1)  // Levenshtein-Zeichen
            {
                $levenshtein = levenshtein($studentSolution, $musterLoesung) / $divisor;
                $similarity_temp = 1 - $levenshtein;
            }
            else if ($answer_distance == 2)  // Soundex-Aussprache
            {
                $levenshtein = levenshtein(soundex($musterLoesung), soundex($studentSolution));

                if (soundex($musterLoesung) == soundex($studentSolution))
                    $similarity_temp = 0.8;
                else if ($levenshtein == 1)
                    $similarity_temp = 0.6;
                else if ($levenshtein == 2)
                    $similarity_temp = 0.4;
                else if ($levenshtein == 3)
                    $similarity_temp = 0.2;
                else  // $levenshtein == 4
                    $similarity_temp = 0;
            }

            if ($correct_array[$i] == 1)  // correct
            {
                if ($similarity_temp > $similarity)
                {
                    $similarity = $similarity_temp;
                    $safe = $similarity_temp == 1;
                }
            }
            else if ($correct_array[$i] == 0.5)  // half correct
            {
                if ($similarity_temp > $similarity)
                {
                    $similarity = $similarity_temp * 0.5;
                    $safe = $similarity_temp == 1;
                }
            }
            else if ($similarity_temp == 1)  // false
            {
                $m_return[1] = 0;       // no points
                $m_return[2] = 1;
                $m_return['percent'] = 0;
                $m_return['safe']    = 1;
                return $m_return;
            }
        }

        $m_return[1] = $similarity * 100;       // percent
        $m_return[2] = $safe;
        $m_return['percent'] = $similarity * 100;
        $m_return['safe']    = $safe;
        return $m_return;
    }



    /**
     *    This function gens a Solution, that is stored to the database.
     *    solution_id.
     *
     *    @access public
     *    @returns XML contains the solution coded in an XML-string
     */

    function genSolution($exercise_id, $p_number_of_answers = null) {

        $msg = '<solution id="'.$exercise_id.'">';

        //same as tb_exercise
        $msg .= '<answer id="0">';
        $msg .= '<body>'.htmlspecialchars(vips_get_string_param("answer_0", true)).'</body>';
        $msg .= '</answer>';

        $msg.= '</solution>';
        return vips_filter_xml_string($msg);
    }



    /**
     *    Generate the XML-input for the database for an exercise.
     *
     *    @abstract
     *    @access public
     *    @returns bool
     */

    function getExerciseConstruct()
    {
        $xmlop  = "<TestItem ID=\"" . $this->type . "\" Type=\"MultipleChoice\" Language=\"" . $this->language . "\">";
        $xmlop .= "<Name>".htmlspecialchars($this->name)."</Name>";
        $xmlop .= "<Hint>".htmlspecialchars($this->hint)."</Hint>";
        $xmlop .= "<CommentField Visible=\"".$this->commentAllowed."\"></CommentField>";
        $xmlop .= "<AnswerDistance>".htmlspecialchars($this->answerDistance)."</AnswerDistance>";
        $xmlop .= "<Question>";
        $xmlop .= "<Paragraph>".htmlspecialchars($this->question)."</Paragraph>";
        $xmlop .= "</Question>";

        for ($i = 0; $i < count($this->answerArray); $i++) {
            $xmlop .= '<Answer Correct="'.$this->correctArray[$i].'">'.htmlspecialchars($this->answerArray[$i]).'</Answer>';
        }
        $xmlop .= "</TestItem>";

        return vips_filter_xml_string($xmlop);
    }

    /**
     * Export this exercise to plain text format.
     */
    function exportText ()
    {
        $result = parent::exportText('Frage');

        if ($this->answerDistance == 1)
            $result .= "Abgleich: Levenshtein\n";
        else if ($this->answerDistance == 2)
            $result .= "Abgleich: Soundex\n";

        for ($i = 0; $i < count($this->answerArray); ++$i)
        {
            if ($this->correctArray[$i] == '1')
                $result .= '+';
            else if ($this->correctArray[$i] == '0.5')
                $result .= '~';

            $result .= 'Antwort: '.$this->answerArray[$i]."\n";
        }

        return $result;
    }



    /**
     * Creates a template for editing a lt_exercise.
     *
     * @returns The template for editing this exercise
     */
    function getEditTemplate()
    {
        global $vipsTemplateFactory;

        if (vips_get_submit_param('add_answer')) {  // add a new answer field
            $number_of_answers = vips_get_int_param('number_of_answers');
            $number_of_answers += 1;
        } else {
            $number_of_answers = $this->numberOfAnswers;
        }

        $template = $vipsTemplateFactory->open('exercises/edit_lt_exercise');
        $template->set_attributes(array(
            'number_of_answers' => $number_of_answers,
            'answer_array'      => $this->getAnswerArray(),
            'correct_array'     => $this->getCorrectArray(),
            'exercise_id'       => vips_get_int_param('exercise_id'),
            'answer_distance'   => $this->answerDistance));

        return $template;
    }



    /**
     * Creates a template for solving an lt_exercise.
     *
     * @returns The template
     */
    function getSolveTemplate($solution_xml = null)
    {
        global $vipsTemplateFactory;

        if (isset($solution_xml)) {
            $student_array = $this->getTagsFromXML($solution_xml, 'answer');
        }

        $template = $vipsTemplateFactory->open('exercises/solve_lt_exercise');
        $template->set_attributes(array(
            'exercise_question' => $this->question,
            'exercise_hint'     => $this->hint,
            'student_array'     => $student_array));

        return $template;
    }



    function getWikiTemplate($solution_xml = null)
    {
        global $vipsTemplateFactory;

        if (isset($solution_xml)) {
            $student_array = $this->getTagsFromXML($solution_xml, 'answer');
        }

        $template = $vipsTemplateFactory->open('exercises/wiki_lt_exercise');
        $template->set_attributes(array(
            'name'          => $this->name,
            'question'      => $this->question,
            'hint'          => $this->hint,
            'student_array' => $student_array
            ));

        return $template;
    }



    /**
     * Builds a template showing a corrected solution.
     *
     * @param $solution_xml The XML string of the student solution
     * @returns The template object
     */
    function getCorrectedWikiTemplate($solution_xml)
    {
        global $vipsTemplateFactory;

        $student_array = $this->getTagsFromXML($solution_xml, 'answer');

        $sim = $this->computeSimilarity($solution_xml);
        $closest_answer = $sim['closest_answer'];
        $closeness      = $sim['closeness'];
        $similarity     = $sim['similarity'];

        $template = $vipsTemplateFactory->open('exercises/wiki_lt_exercise_corrected');
        $template->set_attributes(array(
            'name'           => $this->name,
            'question'       => $this->question,
            'hint'           => $this->hint,
            'answer_array'   => $this->answerArray,
            'correct_array'  => $this->correctArray,
            'student_array'  => $student_array,
            'closest_answer' => $closest_answer,
            'closeness'      => $closeness,
            'threshold'      => 0.75,  // a closeness over this threshold will be displayed as correct
            'similarity'     => $similarity
            ));

        return $template;
    }







    ///////////////////////////////////////////
    ///                                     ///
    ///   H E L P E R   F U N C T I O N S   ///
    ///                                     ///
    ///////////////////////////////////////////



    function answerArrayFromInput()
    {
        $count = vips_get_int_param('number_of_answers');
        $answer_array = array();

        for ($i = 0; $i < $count; ++$i) {
            $answer = vips_get_string_param('answer_'.$i, true);

            if ($answer != '') {
                $answer_array[] = $answer;
            }
        }

        return $answer_array;
    }



    function correctArrayFromInput()
    {
        $count = vips_get_int_param('number_of_answers');
        $correct_array = array();

        for ($i = 0; $i < $count; ++$i) {
            $answer = vips_get_string_param('answer_'.$i, true);

            if ($answer != '') {
                $correct_array[] = vips_get_float_param('correct_'.$i);
            }
        }

        return $correct_array;
    }



    /**
     * Computes the similarity between a given student solution and the correct
     * solutions.
     *
     * @param $solution_xml The student solution XML
     * @returns An associative array with the elements 'closest_answer' (index
     *          of closest answer in answer array), 'closeness' (distance
     *          between student answer and correct answer, in [0;1]) and
     *          similarity (the closeness "weighted" by the answer's
     *          "correctness" (ie. whether the answer was right, half right or
     *          wrong)
     */
    function computeSimilarity($solution_xml)
    {
        $student_array  = $this->getTagsFromXML($solution_xml, 'answer');
        $closest_answer = null;
        $closeness      = 0;

        // answer distance: none
        if ($this->answerDistance == 0) {
            $key = array_search($student_array[0], $this->answerArray);
            if ($key !== false) {
                $closest_answer = $key;
                $closeness      = 1;
            }
        }

        // answer distance: Levenshtein
        else if ($this->answerDistance == 1) {
            $min_lev = strlen(implode('', $this->answerArray)) + 1;  // count all chars in answer_array

            for ($i = 0; $i < count($this->answerArray); $i++) {
                $divisor     = max(strlen($student_array[0]), strlen($this->answerArray[$i]));
                $levenshtein = levenshtein($student_array[0], $this->answerArray[$i]) / $divisor;

                if ($levenshtein == 0) {  // exact match
                    $closest_answer = $i;
                    $closeness      = 1;
                    break;
                }

                if ($levenshtein < $min_lev) {  // found better match than hitherto
                    $min_lev        = $levenshtein;
                    $closest_answer = $i;
                    $closeness      = 1 - $levenshtein;
                }
            }
        }

        // answer distance: soundex
        else if ($this->answerDistance == 2) {
            $min_lev = 5;  // four character soundex values are compared --> levenshtein values are <= 4!

            for ($i = 0; $i < count($this->answerArray); $i++) {
                if ($student_array[0] == $this->answerArray[$i]) {  // identical answers
                    $closest_answer = $i;
                    $closeness      = 1;
                    break;
                }

                // compare soundex values via levenshtein
                $soundex_student = soundex($student_array[0]);  // soundex value: always four characters!
                $soundex_answer  = soundex($this->answerArray[$i]);
                $levenshtein     = levenshtein($soundex_student, $soundex_answer);  // range [0;4]

                if ($levenshtein < $min_lev) {  // found better match than hitherto
                    $min_lev        = $levenshtein;
                    $closest_answer = $i;
                    $closeness      = (0.8 - 0.2 * $levenshtein);  // 0.8 for lev. 0 (same soundex value); 0.6 for lev. 1; 0.4 for lev. 2; 0.2 for lev. 3; 0 for lev. 4
                }
            }
        }

        $similarity = $closeness * $this->correctArray[$closest_answer];

        return array(
            'closest_answer' => $closest_answer,
            'closeness'      => $closeness,
            'similarity'     => $similarity * 100
            );
    }

}
?>
