<?php
/*
 * rnd_exercise.php - Vips plugin for Stud.IP
 * Copyright (c) 2006-2008  Elmar Ludwig, Martin Schrder
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 */

require_once 'exercises/exercise.php';

/**
 * "Random" exercise. Randomly chooses an exercise from a specified list of
 * "subexercises".
 *
 * @author Martin Schrder
 * @copyright virtUOS, 2004-2008
 */
class rnd_exercise extends Exercise
{
    var $subexercises;  // subexercise ids

    // constructor
    function rnd_exercise($xml = null)
    {
        parent::Exercise($xml);

        $this->type = 'rnd_exercise';

        if (!isset($xml)) {  // construct exercise using request parameters
            $this->subexercises = vips_get_int_array_param('stored_subexercises');
        }

        else {  // construct exercise using XML
            $this->subexercises = $this->getAttributesFromXML($xml, 'SubExercise', 'ID');
        }
    }



    function getQuirks()
    {
        global $g_outlet, $vips_db, $g_assignment_table, $g_inAssignment_table;

        // construct $subexercises_array
        $subexercises_array = array();

        if (!empty($this->subexercises)) {
            $sql = "SELECT ID, Name, URI FROM vips_aufgabe WHERE ";
            for ($i = 0; $i < count($this->subexercises); $i++) {
                if ($i > 0) {
                    $sql .= " OR ";
                }
                $sql .= "ID = '".$this->subexercises[$i]."'";
            }
            $result = $vips_db->executeQuery($sql);

            while ($result->next()) {
                $subexercises_array[] = array(
                    'id'   => $result->getInt('ID'),
                    'name' => $result->getString('Name'),
                    'type' => $result->getString('URI'));
            }
        }

        // FIXME this really should only be done when constructing the edit template

        if (isset($g_assignment_table) && isset($g_inAssignment_table)) {
            // fetch all exercise types
            // don't let the lecturer choose a rnd_exercise as a subexercise of a rnd_exercise
            $sql = "SELECT Typenname, URI FROM vips_aufgaben_typen WHERE URI != 'rnd_exercise'";

            $result = $vips_db->executeQuery($sql);
            while ($result->next()) {
                $exercise_types[] = array(
                    'Name' => $result->get('Typenname'),
                    'URI'  => $result->get('URI')
                );
            }

            // fetch all already existing exercises in this course
            // don't let the lecturer choose a rnd_exercise as a subexercise of a rnd_exercise
            $sql = "SELECT
                        $g_assignment_table.Name AS test_name,
                        vips_aufgabe.Name,
                        vips_aufgabe.ID
                    FROM
                        $g_assignment_table,
                        $g_inAssignment_table,
                        vips_aufgabe
                    WHERE
                        $g_assignment_table.Kurs = '$g_outlet->courseID' AND
                        $g_inAssignment_table.$g_assignment_table = $g_assignment_table.ID AND
                        vips_aufgabe.ID = $g_inAssignment_table.vips_aufgabe AND
                        vips_aufgabe.URI != 'rnd_exercise'
                    ORDER BY
                        Beginn, $g_assignment_table.Position, $g_inAssignment_table.Position";

            $result = $vips_db->executeQuery($sql);
            while ($result->next()) {
                $existing_exercises[] = array(
                    'test_name' => $result->get('test_name'),
                    'Name' => $result->get('Name'),
                    'ID' => $result->get('ID')
                );
            }
        }

        return array(
            'subexercises_array' => $subexercises_array,
            'exercise_types'     => $exercise_types,
            'existing_exercises' => $existing_exercises);
    }





    #######################
    #                     #
    #   E V A L U A T E   #
    #                     #
    #######################

    /**
     * Link to subexercise's evaluate function that the user gets presented.
     */
    function evaluate($p_exercise_XML, $p_solution_id, $p_user_id, $p_points)
    {
        global $auth, $vips_db;

        // fetch user id
        $user_id = ($extparam = vips_get_option_param("user_id")) ? $extparam : $auth->auth["uid"];

        // the id of the random exercise is bergeben as POST variable in solutions->correct_uncorrected_solutions()
        $exercise_id = vips_get_int_param("exercise_id");

        // choose one of the subexercises
        $subexercise_id = $this->choose_subexercise($p_exercise_XML, $user_id, $exercise_id);

        $query = "SELECT Aufgabe, URI
            FROM vips_aufgabe
            WHERE ID = '$subexercise_id'
            ";
        $result = $vips_db->executeQuery($query);
        $result->next();
        $subexercise_XML = $result->get("Aufgabe");
        $subexercise_type = $result->get("URI");

        require_once "exercises/$subexercise_type.php";
        $o_class = new $subexercise_type;
        $o_class->setEvaluationMode($this->getEvaluationMode());

        if ($subexercise_type == "pl_exercise")
        {
            return $o_class->evaluate($subexercise_XML, $p_solution_id, $p_user_id, $p_points);
        }
        else
        {
            return $o_class->evaluate($subexercise_XML, $p_solution_id);
        }
    }



    function getSolution($exercise_id, $solution_id)
    {
    }



    function genSolution($p_exercise_id, $p_number_of_answers = null)
    {
        global $auth;

        // fetch user id
        $m_user_id = ($extparam = vips_get_option_param("user_id")) ? $extparam : $auth->auth["uid"];

        // choose one of the subexercises
        $m_subexercise_array = $this->chooseSubexercise($m_user_id, $p_exercise_id);

        $m_chosen_subexercise_id = $m_subexercise_array["id"];
        $m_chosen_subexercise_type = $m_subexercise_array["type"];
        $m_0class = new $m_chosen_subexercise_type;
        return $m_0class->genSolution($m_chosen_subexercise_id, $p_number_of_answers);
    }





    function setSolution($exercise_id, $solution_id)
    {
    }

    function getStatistic($exercise_id, $course_id)
    {
    }



    /////////////////////////////////////////////////////
    //                                                 //
    //   G E T   E X E R C I S E   C O N S T R U C T   //
    //                                                 //
    /////////////////////////////////////////////////////

    /**
     * This function computes the XML value to represent the exercise.
     */
    function getExerciseConstruct()
    {
        $xml = '<TestItem ID="'.$this->type.'" Type="RandomExercise" Language="'.$this->language.'">'
            .'<Name>'.htmlspecialchars($this->name).'</Name>'
            .'<CommentField Visible="'.$this->commentAllowed.'"></CommentField>';

        // every subexercise is put into XML //
        foreach ($this->subexercises as $sub_id) {
            $xml .= '<SubExercise ID="'.$sub_id.'"/>';
        }

        // add new subexercise //
        if (vips_get_submit_param('add')) {
            $exercise_to_add = vips_get_int_param('existing_exercises');  // from dropdown menu

            if ($exercise_to_add != 'NULL') {
                $xml .= '<SubExercise ID="'.$exercise_to_add.'"/>';
            }
        }

        $xml .= '</TestItem>';

        return vips_filter_xml_string($xml);
    }



    function getEditTemplate()
    {
        global $vipsTemplateFactory, $vips_db, $g_outlet;
        global $g_assignment_table, $g_inAssignment_table;

        $assignment_id     = vips_get_int_param('assignment_id');
        $exercise_position = vips_get_int_param('exercise_position');

        $query = "SELECT vips_aufgabe.*
                  FROM vips_aufgabe, $g_inAssignment_table
                  WHERE $g_inAssignment_table.$g_assignment_table = $assignment_id
                      AND $g_inAssignment_table.Position = $exercise_position
                      AND $g_inAssignment_table.vips_aufgabe = vips_aufgabe.ID";

        $result = $vips_db->executeQuery($query);

        if ($result->next()) {
            $exercise_id   = $result->getInt('ID');
            $exercise_name = $result->getString('Name');
        }

        $template = $vipsTemplateFactory->open('exercises/edit_rnd_exercise');
        $template->set_attributes(array(
            'subexercises_array'    => $this->getQuirk('subexercises_array'),
            'action'                => vips_get_string_param('action'),
            'assignment_id'         => $assignment_id,
            'exercise_id'           => $exercise_id,
            'exercise_name'         => $exercise_name,
            'exercise_position'     => $exercise_position,
            'picturepath'           => $g_outlet->picturepath,
            'existing_exercises'    => $this->getQuirk('existing_exercises'),
            'exercise_types'        => $this->getQuirk('exercise_types'))
        );

        return $template;
    }



    /**
     * Creates a template for solving an rnd_exercise.
     *
     * @returns The template
     */
    function getSolveTemplate($solution_xml = null)
    {
        global $g_outlet, $vips_db;

        // fetch subexercise id
        $exercise_xml   = $this->getExerciseConstruct();
        $solver_user_id = vips_get_string_param('solver_user_id');
        $user_id        = isset($solver_user_id) ? $solver_user_id : $g_outlet->userID;
        $exercise_id    = vips_get_int_param('exercise_id');
        $subexercise_id = $this->choose_subexercise($exercise_xml, $user_id, $exercise_id);

        // get and construct subexercise and return its solve template
        $sql = "SELECT Aufgabe, URI FROM vips_aufgabe WHERE ID = '$subexercise_id'";
        $result = $vips_db->executeQuery($sql);
        if ($result->next()) {
            $subexercise_xml  = $result->getString('Aufgabe');
            $subexercise_type = $result->getString('URI');

            $_POST['exercise_id']     = $subexercise_id;
            $_POST['rnd_exercise_id'] = $exercise_id;
            include_once("exercises/$subexercise_type.php");
            $subexercise = new $subexercise_type($subexercise_xml);

            return $subexercise->getSolveTemplate($solution_xml);
        }
    }





    ###################################################
    #                                                 #
    #   A P P E N D   N E W   S U B E X E R C I S E   #
    #                                                 #
    ###################################################

    /**
     * Appends a new subexercise to a given random exercise.
     *
     * @param $p_XML The XML representation of the random exercise as it is constructed by
     *               <cc>getExerciseConstruct</cc>.
     * @param $p_ID A new ID which will be inserted to the end of the given XML string.
     *
     * @returns The new XML string with the new ID appended.
     */
    function appendNewSubexercise($p_XML, $p_ID)
    {
        // Overwrite the end tag ("</TestItem>", 11 characters long!) with the new
        // <SubExercise> tag plus a new "</TestItem>". Result: the new <SubExercise> tag has
        // been inserted.
        $xml_return_value = substr_replace
            ( $p_XML
            , "<SubExercise ID=\"$p_ID\"/></TestItem>"
            , -11
            );

        return $xml_return_value;
    }






    /**
     * Deletes a subexercise from the exercise XML.
     */
    function delete_subexercise($p_xml, $p_id)
    {
        return str_replace("<SubExercise ID=\"$p_id\"/>", "", $p_xml);
    }





    ###########################################
    #                                         #
    #   C H O O S E   S U B E X E R C I S E   #
    #                                         #
    ###########################################

    /**
     * Chooses a subexercise dependent on the user id.
     *
     * @param $p_user_id The user id as hexadecimal md5-hash.
     * @param $p_rnd_exercise_id The random exercise's id.
     * @return The id of the chosen subexercise.
     */
    function choose_subexercise($p_xml, $p_user_id, $p_rnd_exercise_id)
    {
        // Initializing randomizer with specific data (user and exercise dependend) determines
        // the exercise the user is given!
        $randomizer_start = crc32($p_user_id.$p_rnd_exercise_id);
        mt_srand($randomizer_start);

        // chooses a subexercise from the given set of subexercises stored in
        // this random exercise
        $subexercise_ids =
            $this->getAttributesFromXML  // in exercise.php
                ( $p_xml
                , "SubExercise"
                , "ID"
                );
        $subexercises_number = count($subexercise_ids);

        if ($subexercises_number > 0)
        {
            $chosen_index          = mt_rand(0, $subexercises_number - 1);
            $chosen_subexercise_id = $subexercise_ids[$chosen_index];
            return $chosen_subexercise_id;
        }

        return false;
    }
}
?>
