<?php
/*
 * yn_exercise.php - Vips plugin for Stud.IP
 * Copyright (c) 2003-2005  Erik Schmitt, Philipp Hgelmeyer
 * Copyright (c) 2005-2006  Christa Deiwiks
 * Copyright (c) 2006-2008  Elmar Ludwig, Martin Schrder
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 */

require_once 'exercises/exercise.php';

/**
 * XXX detailed description
 *
 * @author    XXX
 * @copyright XXX
 * @abstract
 */
class yn_exercise extends Exercise
{
    var $labelYes,
        $labelNo;

    // constructor
    function yn_exercise($xml = null)
    {
        parent::Exercise($xml);

        $this->type            = 'yn_exercise';
        $this->numberOfAnswers = 2;

        if (!isset($xml)) {  // construct exercise using request parameters
            $this->labelYes       = vips_get_string_param('label_yes', true);
            $this->labelNo        = vips_get_string_param('label_no', true);
            $this->answerArray    = $this->answerArrayFromInput();
            $this->correctArray   = $this->correctArrayFromInput();
        }
        else {  // construct exercise using XML
            $dom_xml = $this->getDomXML($xml);
            $answers = $dom_xml->get_elements_by_tagname('Answer');

            foreach ($answers as $key => $node)
            {
                $this->answerArray[] = utf8_decode($node->get_content());

                if ($node->get_attribute('Correct') == '1')
                    $this->correctArray[0] = $key;
            }

            $this->labelYes = $this->answerArray[0];
            $this->labelNo  = $this->answerArray[1];
        }
    }



    /**
     * Returns all variables which are peculiar for this exercise type.
     *
     * @returns An array with the keys 'label_yes' and 'label_no'
     */
    function getQuirks()
    {
        return array(
            'label_yes' => $this->labelYes,
            'label_no'  => $this->labelNo);
    }



    function setDefaultValues()
    {
        $this->labelYes = _('Ja');
        $this->labelNo = _('Nein');
    }



    /**************************************************************************/
    /*
    /* Evaluates the solution
    /*
    /**************************************************************************/

    /**
     *    Evaluates a solution.
     *
     *    @abstract
     *    @access public
     *    @returns int (from 0 to 100 ) as an % value
     */
    function evaluate($exercise,$solution)
    {
        $punkte = 0;

        $dom_xml = $this->getDomXML($exercise);
        $answers = $dom_xml->get_elements_by_tagname("Answer");

        $answer_array = array();
        $correct_array = array();

        //get answer + correct array
        foreach ($answers as $key=>$node) {
            $answer_array[]         = html_ready(utf8_decode($node->get_content()));
            if ($node->get_attribute("Correct") == '1')
                $correct_array[0]       = $key;
        }
        $student_array = $this->getTagsFromXML($solution, "answer");

        if ($student_array[0] == $correct_array[0]) $punkte = 1;

        $m_prozent = $punkte * 100 ;
        $m_save = 0;
        if ($m_prozent == 100 || $m_prozent == 0) {
            $m_save = 1;
        }

        $m_return[1] = $m_prozent;
        $m_return[2] = $m_save;
        $m_return[5] = $punkte;
        $m_return['percent'] = $m_prozent;
        $m_return['safe']    = $m_save;
        return $m_return;
    }



    /**************************************************************************/
    /*
    /* XML-generating routines
    /*
    /**************************************************************************/



    /**
     * Generates an XML-String containing the solution of the student.
     *
     * Gets its values to be stored in the XML-String from the
     * Get/Post environment.
     *
     * @param int $exercise_id the id of the exercise the solution related to.
     * @param int $p_number_of_answers the number of answers possible (??? not a class variable, ES)
     * @return XML the XML-String containing the solutions of the student.
     *
     * @todo names of the exml entities are not well choosen. A change will have
     * impact on other parts of the code, of course.
     */

    function genSolution($p_exercise_id, $p_number_of_answers = null)
    {
        $xml  = '<solution id="'.$p_exercise_id.'">';
        $xml .= '<answer id="0">';
        $xml .= '<body>'.htmlspecialchars(vips_get_int_param("answer_0")).'</body>';  //position!
        $xml .= '</answer>';
        $xml .= '</solution>';

        return vips_filter_xml_string($xml);
    }



    /**
     * Generate the description of an exercise in an XML-String.
     *
     * Gets its values to be stored in the XML-String from the
     * Get/Post environment.
     *
     * @return XML the XML-String describing the exercise in terms of its
     * name, questions, wrong and correct solutions etc.
     */

    function getExerciseConstruct()
    {
        $xmlop  = '<TestItem ID="'.$this->type.'" Type="JaNeinFrage" Language="'.$this->language.'">';
        $xmlop .= "<Name>".htmlspecialchars($this->name)."</Name>";
        $xmlop .= "<Hint>".htmlspecialchars($this->hint)."</Hint>";
        $xmlop .= '<CommentField Visible="'.$this->commentAllowed.'"></CommentField>';
        $xmlop .= "<Question>";
        $xmlop .= "<Paragraph>".htmlspecialchars($this->question)."</Paragraph>";
        $xmlop .= "</Question>";

        $index = $this->correctArray[0];
        for ($i = 0; $i < count($this->answerArray); $i++) {
            if ($this->answerArray[$i]) {
                if ($i == $index) {
                    $xmlop .= '<Answer Correct="1">'.htmlspecialchars($this->answerArray[$i]).'</Answer>';
                }
                else {
                    $xmlop .= '<Answer Correct="0">'.htmlspecialchars($this->answerArray[$i]).'</Answer>';
                }

            }
        }

        $xmlop .= "</TestItem>";

        return vips_filter_xml_string($xmlop);
    }

    /**
     * Export this exercise to plain text format.
     */
    function exportText ()
    {
        $result = parent::exportText('JN-Frage');

        for ($i = 0; $i < count($this->answerArray); ++$i)
        {
            if ($this->correctArray[0] == $i)
                $result .= '+';

            $result .= 'Antwort: '.$this->answerArray[$i]."\n";
        }

        return $result;
    }



    /**
     * Creates a template for editing a yn_exercise.
     *
     * @returns The template for editing this exercise
     */
    function getEditTemplate()
    {
        global $vipsTemplateFactory;

        $template = $vipsTemplateFactory->open('exercises/edit_yn_exercise');
        $template->set_attributes(array(
            'correct_array' => $this->getCorrectArray(),
            'label_yes'     => $this->labelYes,
            'label_no'      => $this->labelNo));

        return $template;
    }



    /**
     * Creates a template for solving a yn_exercise.
     *
     * @returns The template
     */
    function getSolveTemplate($solution_xml = null)
    {
        global //$g_outlet,
               $vipsTemplateFactory;

//        $solver_user_id    = vips_get_string_param('solver_user_id');
//        $user_id           = isset($solver_user_id) ? $solver_user_id : $g_outlet->userID;
//        $exercise_position = vips_get_int_param('exercise_position');
//        $assignment_id     = vips_get_int_param('assignment_id');
//        $assignment_type   = vips_get_string_param('action') == 'sheets' ? 'vips_uebungsblatt' : 'vips_klausur';
//        $solution_xml      = getUserSolution($user_id, $exercise_position, $assignment_id, $assignment_type);
        if (isset($solution_xml)) {
            $student_array = $this->getTagsFromXML($solution_xml, 'answer');
        }

        $template = $vipsTemplateFactory->open('exercises/solve_yn_exercise');
        $template->set_attributes(array(
            'exercise_question' => $this->question,
            'exercise_hint'     => $this->hint,
            'student_array'     => $student_array,
            'label_yes'         => $this->labelYes,
            'label_no'          => $this->labelNo));

        return $template;
    }



    /**
     * Fills a wiki template with all necessary values. If, additionally, a
     * solution is processed to it, it will also be set in.
     *
     * @param $solution (optional) The XML string of the student solution
     * @return The filled wiki template object
     */
    function getWikiTemplate($solution_xml = null)
    {
        global $vipsTemplateFactory;

        if (isset($solution_xml)) {
            $student_array = $this->getTagsFromXML($solution_xml, 'answer');
        }

        $template = $vipsTemplateFactory->open('exercises/wiki_yn_exercise');
        $template->set_attributes(array(
            'name'          => $this->name,
            'question'      => $this->question,
            'label_yes'     => $this->labelYes,
            'label_no'      => $this->labelNo,
            'hint'          => $this->hint,
            'answer_array'  => $this->answerArray,
            'student_array' => $student_array));

        return $template;
    }



    /**
     * Builds a template showing a corrected solution.
     *
     * @param $solution_xml The XML string of the student solution
     * @returns The template object
     */
    function getCorrectedWikiTemplate($solution_xml)
    {
        global $vipsTemplateFactory;

        $student_array = $this->getTagsFromXML($solution_xml, 'answer');

        $template = $vipsTemplateFactory->open('exercises/wiki_yn_exercise_corrected');
        $template->set_attributes(array(
            'name'          => $this->name,
            'question'      => $this->question,
            'label_yes'     => $this->labelYes,
            'label_no'      => $this->labelNo,
            'hint'          => $this->hint,
            'answer_array'  => $this->answerArray,
            'correct_array' => $this->correctArray,
            'student_array' => $student_array));

        return $template;
    }



    /**
     * Constructs the <tt>answer array</tt> from the request environment.
     *
     * @returns The answer array
     */
    function answerArrayFromInput()
    {
        return array(
            vips_get_string_param('label_yes', true),
            vips_get_string_param('label_no', true));
    }



    /**
     * Constructs the <tt>correct array</tt> from the request environment.
     *
     * @returns The correct array
     */
    function correctArrayFromInput()
    {
        return array(vips_get_int_param('correct_0'));
    }
}
?>
