<?php
/*
 *  $Id: MySQLConnection.php,v 1.18 2004/09/01 14:00:28 dlawson_mi Exp $
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many individuals
 * and is licensed under the LGPL. For more information please see
 * <http://creole.phpdb.org>.
 */
 
require_once 'creole/Connection.php';
require_once 'creole/common/ConnectionCommon.php';
include_once 'creole/drivers/mysql-pdo/MySQLResultSet.php';

function mysql_pdo_error($pdo)
{
    $error_info = $pdo->errorInfo();

    return $error_info[1];
}

function mysql_pdo_select_db($pdo, $dbname)
{
    return true;
}

/**
 * MySQL implementation of Connection.
 * 
 * 
 * @author    Hans Lellelid <hans@xmpl.org>
 * @author    Stig Bakken <ssb@fast.no> 
 * @author    Lukas Smith
 * @version   $Revision: 1.18 $
 * @package   creole.drivers.mysql
 */ 
class MySQLConnection extends ConnectionCommon implements Connection {

    /** Current database (used in mysql_select_db()). */
    private $database;
    
    private $updateCount;

    /**
     * Connect to a database and log in as the specified user.
     *
     * @param $dsn the data source name (see DB::parseDSN for syntax)
     * @param $flags Any conneciton flags.
     * @access public
     * @throws SQLException
     * @return void
     */
    function connect($dsninfo, $flags = 0)
    {
        $this->dsn = $dsninfo;
        $this->flags = $flags;
        
        $this->dblink = DBManager::get();
    }    
    
    /**
     * @see Connection::getDatabaseInfo()
     */
    public function getDatabaseInfo()
    {
        require_once 'creole/drivers/mysql-pdo/metadata/MySQLDatabaseInfo.php';
        return new MySQLDatabaseInfo($this);
    }
    
    /**
     * @see Connection::getIdGenerator()
     */
    public function getIdGenerator()
    {
        require_once 'creole/drivers/mysql-pdo/MySQLIdGenerator.php';
        return new MySQLIdGenerator($this);
    }
    
    /**
     * @see Connection::prepareStatement()
     */
    public function prepareStatement($sql) 
    {
        require_once 'creole/drivers/mysql-pdo/MySQLPreparedStatement.php';
        return new MySQLPreparedStatement($this, $sql);
    }
    
    /**
     * @see Connection::prepareCall()
     */
    public function prepareCall($sql) {
        throw new SQLException('MySQL does not support stored procedures.');
    }
    
    /**
     * @see Connection::createStatement()
     */
    public function createStatement()
    {
        require_once 'creole/drivers/mysql-pdo/MySQLStatement.php';
        return new MySQLStatement($this);
    }
        
    /**
     * @see Connection::disconnect()
     */
    function close()
    {
        $ret = true;
        $this->dblink = null;
        return $ret;
    }
    
    /**
     * @see Connection::applyLimit()
     */
    public function applyLimit(&$sql, $offset, $limit)
    {
        if ( $limit > 0 ) {
            $sql .= " LIMIT " . ($offset > 0 ? $offset . ", " : "") . $limit;
        } else if ( $offset > 0 ) {
            $sql .= " LIMIT " . $offset . ", 18446744073709551615";
        }
    }

    /**
     * @see Connection::executeQuery()
     */
    function executeQuery($sql, $fetchmode = null)
    {
        $this->lastQuery = $sql;
        if ($this->database) {
            if (!mysql_pdo_select_db($this->dblink, $this->database)) {
                throw new SQLException('No database selected', $this->dblink->errorInfo());
            }
        }
        $result = @$this->dblink->query($sql);
        if (!$result) {
            throw new SQLException('Could not execute query', $this->dblink->errorInfo(), $sql);
        }
        return new MySQLResultSet($this, $result, $fetchmode);
    }
    
    /**
     * @see Connection::executeUpdate()
     */
    function executeUpdate($sql)
    {    
        $this->lastQuery = $sql;

        if ($this->database) {
            if (!mysql_pdo_select_db($this->dblink, $this->database)) {
                    throw new SQLException('No database selected', $this->dblink->errorInfo());
            }
        }
        
        $result = @$this->dblink->exec($sql);
        if ($result === false) {
            throw new SQLException('Could not execute update', $this->dblink->errorInfo(), $sql);
        }        
        return $this->updateCount = $result;
    }

    /**
     * Start a database transaction.
     * @throws SQLException
     * @return void
     */
    protected function beginTrans()
    {
        $result = @$this->dblink->exec('SET AUTOCOMMIT=0');
        $result = @$this->dblink->exec('BEGIN');
        if ($result === false) {
            throw new SQLException('Could not begin transaction', $this->dblink->errorInfo());
        }
    }
        
    /**
     * Commit the current transaction.
     * @throws SQLException
     * @return void
     */
    protected function commitTrans()
    {
        if ($this->database) {
            if (!mysql_pdo_select_db($this->dblink, $this->database)) {
                 throw new SQLException('No database selected', $this->dblink->errorInfo());
            }
        }
        $result = @$this->dblink->exec('COMMIT');
        $result = @$this->dblink->exec('SET AUTOCOMMIT=1');
        if ($result === false) {
            throw new SQLException('Can not commit transaction', $this->dblink->errorInfo());                
        }
    }

    /**
     * Roll back (undo) the current transaction.
     * @throws SQLException
     * @return void
     */
    protected function rollbackTrans()
    {
        if ($this->database) {
            if (!mysql_pdo_select_db($this->dblink, $this->database)) {
                throw new SQLException('No database selected', $this->dblink->errorInfo());
            }
        }
        $result = @$this->dblink->exec('ROLLBACK');
        $result = @$this->dblink->exec('SET AUTOCOMMIT=1');
        if ($result === false) {
            throw new SQLException('Could not rollback transaction', $this->dblink->errorInfo());
        }
    }

    /**
     * Gets the number of rows affected by the data manipulation
     * query.
     *
     * @return int Number of rows affected by the last query.
     */
    function getUpdateCount()
    {
        return $this->updateCount;
    }
    
}
