<?php
/*
 * vips_groups.inc.php - Vips plugin for Stud.IP
 * Copyright (c) 2003-2005  Erik Schmitt, Philipp Hgelmeyer
 * Copyright (c) 2005-2006  Christa Deiwiks
 * Copyright (c) 2006-2007  Elmar Ludwig, Martin Schrder
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 */

/**
 *
 * Handles Uebungsgruppen in Vips (different from groups in Stud.IP). A group is a number of
 * students who work together on an assignment. The last solution submitted by any of the members
 * counts. Membership in a group is marked by start and end date, so when a student switches
 * groups, (s)he gets the correct points in the end.
 *
 * @author: Christa Deiwiks <cdeiwiks@uos.de>
 */

function vips_groups ($subaction)
{
    global $g_outlet;

    if ($g_outlet->status == 'lecturer' || $g_outlet->status == 'tutor') {
        if ($subaction == 'edit_group') {
            edit_group();
        } else if ($subaction == 'store_group') {
            store_group();
        } else if ($subaction == 'delete_group') {
            delete_group();
        } else if ($subaction == 'store_access') {
            store_access();
        } else if ($subaction == 'assign_participant') {
            assign_participant();
        } else if ($subaction == 'delete_participant') {
            delete_participant();
        } else {
            main();
        }
    } else {
        if ($subaction == 'assign_participant') {
            assign_participant();
        } else if ($subaction == 'delete_participant') {
            delete_participant();
        } else {
            main();
        }
    }
}


 /**
  * Displays main page
  * STUDENTS: see the groups and can possibly assign themselves to a group (optional by lecturer)
  * LECTURERS: can create groups and assign any student to any group
  *
  */
function main() {
        global $vips_db, $g_outlet, $vipsTemplateFactory;

        $t_main = $vipsTemplateFactory->open('groups_main');
        $t_main->set_attributes(array(
                'picturepath' => $g_outlet->picturepath,
                'status' => $g_outlet->status));

        #################################
        #  get messages and dialog info #
        #################################

        $message  = vips_get_string_array_param('message');
        $dialog   = vips_get_string_param('dialog');
        $group_id = vips_get_int_param('group_id');

        $t_main->set_attribute('message',  $message);
        $t_main->set_attribute('dialog',   $dialog);
        $t_main->set_attribute('group_id', $group_id);

        #################################
        #  get selfassign information   #
        #################################

        $selfassign = 0;
        $result = $vips_db->executeQuery("SELECT * FROM vips_optionen
                                          WHERE Kursid = '$g_outlet->courseID'");
        if ($result->next()) {
                $optionen = $result->getRow();
                $selfassign = $optionen['Selbstzuweisung'];
        }
        $t_main->set_attribute('selfassign', $selfassign);

        #################################
        #  if I come from edit_group()  #
        #################################
        $edit_group = false;                                                            //switch for title: "Gruppe anlegen" vs "GRuppe editieren"
        if (vips_get_int_param('edit_group')) {
                $group_name = vips_get_string_param('group_name');
                $group_size = vips_get_int_param('group_size');
                $group_id   = vips_get_int_param('group_id');

                $t_main->set_attribute('group_name', $group_name);
                $t_main->set_attribute('group_size', $group_size);
                $t_main->set_attribute('group_id', $group_id);

                $edit_group = true;

        }

        #################################
        #  get all course participants  #
        #################################

        if ($g_outlet->status == 'lecturer' || $g_outlet->status == 'tutor') {
            $user_filter = '';
        } else {
            $user_filter = "AND seminar_user.user_id = '$g_outlet->userID'";
        }

        $course_participants = array();
        $result = $vips_db->executeQuery("SELECT seminar_user.*,
                                                 auth_user_md5.username,
                                                 auth_user_md5.Vorname,
                                                 auth_user_md5.Nachname
                                          FROM  seminar_user, auth_user_md5
                                          WHERE seminar_user.Seminar_id = '$g_outlet->courseID'
                                            AND seminar_user.user_id    = auth_user_md5.user_id
                                            AND seminar_user.status     != 'dozent'
                                            AND seminar_user.status     != 'tutor'
                                            $user_filter
                                          ORDER BY auth_user_md5.Nachname, auth_user_md5.Vorname");
        while ($result->next()) {
                $course_participant = $result->getRow();

                $user_id = $course_participant['user_id'];
                $result2 = $vips_db->executeQuery("SELECT vips_inGruppe.* FROM
                                                          vips_inGruppe,
                                                          vips_gruppe
                                                    WHERE user_id = '$user_id'
                                                      AND vips_gruppe.Gruppenid = vips_inGruppe.group_id
                                                      AND vips_gruppe.Kursid    = '$g_outlet->courseID'
                                                      AND vips_inGruppe.end IS NULL");
                if ($result2->next())
                        $course_participant['group_id'] = $result2->getInt('group_id');
                else
                        $course_participant['group_id'] = 0;

                $course_participants[] = $course_participant;
        }
        $t_main->set_attribute('course_participants', $course_participants);


        #############################################################
        #  get all groups and their participants (multidim array)  #
        #############################################################

        $groups = array();

        $result = $vips_db->executeQuery("SELECT *
                                            FROM vips_gruppe
                                            WHERE Kursid = '$g_outlet->courseID'
                                            ORDER BY Gruppenid");
        while ($result->next()) {
                $group = $result->getRow();
                $group_participants = array();
                $group_id = $group['Gruppenid'];

                $result2 = $vips_db->executeQuery("SELECT auth_user_md5.user_id,
                                                          auth_user_md5.username,
                                                          auth_user_md5.Vorname,
                                                          auth_user_md5.Nachname
                                                    FROM  vips_inGruppe,
                                                          auth_user_md5
                                                    WHERE vips_inGruppe.group_id = '$group_id'
                                                      AND vips_inGruppe.user_id = auth_user_md5.user_id
                                                      AND vips_inGruppe.end IS NULL
                                                    ORDER BY Nachname, Vorname");   //get current group (membership hasn't ended yet)

                while ($result2->next())
                {
                        $group_participant = $result2->getRow();
                        $group_participants[] = $group_participant;
                }

                $group['participants'] = $group_participants;
                $groups[] = $group;
        }
        $t_main->set_attribute('groups', $groups);
        $t_main->set_attribute('number_of_groups', count($groups));
        $t_main->set_attribute('edit_group', $edit_group); //true or false
        $t_main->set_attribute('user_id', $g_outlet->userID);

        ############
        #  render  #
        ############

        setlocale(LC_NUMERIC, NULL);
        echo $t_main->render();
        setlocale(LC_NUMERIC, 'C');
}



/**
 * Stores a group created by lecturer
 */
function store_group() {
        global $vips_db, $g_outlet;

        $group_name = vips_get_string_param('group_name', true);
        $group_size = vips_get_int_param('group_size');
        $group_id   = vips_get_int_param('group_id');

        // assure that $group_size >= 1
        if ($group_size < 1) $group_size = 1;

        if (vips_get_submit_param('cancel') || vips_get_submit_param('cancel_x') != '') {
                // don't store group
        }
        else if ($group_id) {
                $vips_db->executeUpdate("UPDATE vips_gruppe
                                        SET Gruppenname         = '$group_name',
                                                Kursid                  = '$g_outlet->courseID',
                                                Gruppengroesse  = '$group_size'
                                        WHERE Gruppenid = '$group_id'");
                $message['mode']        = 'msg';
                $message['message'] = "Gruppe wurde gendert.";
        }
        else {
                $result = $vips_db->executeQuery("SELECT * FROM vips_gruppe
                                                                WHERE Gruppenname = '$group_name'
                                                                AND Kursid = '$g_outlet->courseID'
                                                        ");
                if ($result->getRecordCount() == 0) {
                        $vips_db->executeUpdate("INSERT INTO vips_gruppe
                                        SET Gruppenname         = '$group_name',
                                                Kursid                  = '$g_outlet->courseID',
                                                Gruppengroesse  = '$group_size'");
                        $message['mode']        = 'msg';
                        $message['message'] = "Gruppe wurde angelegt.";
                }
                else {
                        $message['mode']        = 'error';
                        $message['message'] = "Eine Gruppe dieses Namens gibt es schon. Bitte whlen Sie einen anderen Namen!";
                }
        }
        $_POST['message'] = $message;
        main();

}

/**
 * Lecturers can change group size and group name
 */
function edit_group() {
        $_POST['edit_group'] = 1;
        main();
}

/**
 * Deletes a group
 */
function delete_group() {
        global $vips_db, $g_outlet;
        $group_id       = vips_get_int_param('group_id');

        if (vips_get_submit_param('delete_group_confirmed') || vips_get_submit_param('delete_group_confirmed_x') != '') {
                $query = "DELETE FROM vips_gruppe WHERE Gruppenid = '$group_id'";
                $vips_db->executeUpdate($query);
                $message['mode']        = 'msg';
                $message['message'] = "Gruppe wurde gelscht.";
        }
        else if (vips_get_submit_param('cancel') == '' && vips_get_submit_param('cancel_x') == '') {
                // fetch people which are and STILL are group members
                $query = "SELECT * FROM vips_inGruppe WHERE group_id = '$group_id' AND end IS NULL";
                $result = $vips_db->executeQuery($query);
                if ($result->getRecordCount() != 0) {
                        $message['mode']    = 'error';
                        $message['message'] = "In der Gruppe befinden sich noch Teilnehmer!";
                }
                else {  // ask to confirm delete
                        $_POST['dialog'] = 'delete_group';
                        $_POST['group_id'] = $group_id;
                }
        }
        $_POST['message']       = $message;
        main();
}

/**
 * Puts a student into a group
 */

function assign_participant() {
        global $vips_db, $g_outlet;

        $number_of_groups = vips_get_int_param('number_of_groups');
        $selected_participants = vips_get_option_array_param('course_participants');

        if (count($selected_participants) == 0) {
                $message['mode']    = 'error';
                $message['message'] = _('Bitte whlen Sie einen oder mehrere Teilnehmer aus der Liste.');
                $_POST['message']   = $message;
        }
        else {
                 for($i = 0; $i < $number_of_groups; ++$i) {

                        if (vips_get_submit_param("assign_to_group_$i")) {

                                $group_id = vips_get_int_param("group_$i");

                                $message = array();

                                foreach ($selected_participants as $user_id) {

                                        $error = false;

                                        $result2 = $vips_db->executeQuery("SELECT * FROM vips_gruppe WHERE Gruppenid = '$group_id'");
                                        $result2->next();
                                        $group = $result2->getRow();
                                        $group_size = $group['Gruppengroesse'];

                                        $result2 = $vips_db->executeQuery("SELECT * FROM vips_inGruppe
                                                                            WHERE group_id = '$group_id'
                                                                            AND end IS NULL");
                                        $number_of_participants = $result2->getRecordCount();

                                        $result = $vips_db->executeQuery("SELECT vips_inGruppe.*
                                                                            FROM vips_inGruppe, vips_gruppe
                                                                            WHERE vips_inGruppe.user_id = '$user_id'
                                                                            AND vips_gruppe.Kursid = '$g_outlet->courseID'
                                                                            AND vips_gruppe.Gruppenid = vips_inGruppe.group_id
                                                                            AND vips_inGruppe.end IS NULL
                                                                ");
                                        //TODO get Status of user_id: Eingetragener != Eintrager;       ???

                                        //check if user_id is valid (=same)
                                        if ($g_outlet->status == "student" && $g_outlet->userID != $user_id) {
                                                $message['mode']    = 'error';
                                                $message['message'] = _('Bitte tragen Sie nur sich selbst in eine Gruppe ein.');
                                                $error = true;
                                        }
                                        // check for membership in this or other groups (tutors are excempt for the former)
                                        elseif ($result->next()) {
                                                $entry = $result->getRow();
                                                if ($entry['group_id'] == $group_id) {
                                                        $message['mode']    = 'error';
                                                        $message['message'] = _('Sie befinden sich schon in dieser Gruppe.');
                                                        $error = true;
                                                }
                                                elseif ($g_outlet->status != 'tutor') {
                                                        $message['mode']    = 'error';
                                                        $message['message'] = _('Teilnehmer befindet sich schon in einer Gruppe.');
                                                        $error = true;
                                                }
                                        }
                                        // check for place availability
                                        elseif ($number_of_participants >= $group_size) {
                                                $message['mode']    = 'error';
                                                $message['message'] = _('Diese Gruppe ist bereits voll.');
                                                $error = true;
                                        }
                                        //TODO: Tutoren sind erlaubt auch wenn Gruppe voll

                                        $now = date("Y-m-d H:i:s");

                                        if (!$error)  {
                                                $vips_db->executeUpdate("INSERT INTO vips_inGruppe
                                                                        SET group_id    = '$group_id',
                                                                            user_id     = '$user_id',
                                                                            start       = '$now'
                                                                ");
                                                $message['mode'] = 'msg';
                                                $message['message'] = "Teilnehmer wurde in die Gruppe eingetragen!";
                                        }
                                } // endforeach selected participants

                        }
                }// endforeach number of groups
        }
        $_POST['message'] = $message;
        main();
}


/**
 * Removes a student/tutor from a group
 */
function delete_participant() {
        global $vips_db, $g_outlet;
        $group_id       = vips_get_int_param('group_id');
        $user_id        = vips_get_option_param('user_id');

        if ($g_outlet->status == "student" && $g_outlet->userID != $user_id) {
                $message['mode']    = 'error';
                $message['message'] = _('Bitte lschen Sie nur sich selbst aus einer Gruppe.');
        }
        else
        {
                $now = date("Y-m-d H:i:s");
                $sql = "UPDATE vips_inGruppe
                        SET end = '$now'
                        WHERE group_id  = '$group_id'
                        AND user_id = '$user_id'
                        AND end IS NULL";
                $vips_db->executeUpdate($sql);

                $message['mode'] = 'msg';
                $message['message'] = _('Teilnehmer wurde aus der Gruppe ausgetragen.');
        }

        $_POST['message'] = $message;
        main();
}


/**
 *
 * Stores student and tutor access to different areas
 * TODO: Extensible, since there are only two options at the moment
 *
 */
function store_access()
{
    global $vips_db, $g_outlet;

    $options = vips_get_int_array_param('options');

    $result = $vips_db->executeQuery("SELECT * FROM vips_optionen WHERE Kursid = '$g_outlet->courseID'");

    if ($result->getRecordCount())
    {
        $vips_db->executeUpdate("UPDATE vips_optionen
                                 SET Selbstzuweisung = '$options[0]',
                                     Tutor           = '$options[1]'
                                 WHERE Kursid = '$g_outlet->courseID'");
    }
    else
    {
        $vips_db->executeUpdate("INSERT INTO vips_optionen
                                 SET Kursid          = '$g_outlet->courseID',
                                     Selbstzuweisung = '$options[0]',
                                     Tutor           = '$options[1]'");
    }

    main();
}
?>
