<?php
/*
 * vips_import.inc.php - Vips plugin for Stud.IP
 * Copyright (c) 2005-2006  Christa Deiwiks
 * Copyright (c) 2006-2007  Elmar Ludwig, Martin Schrder
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 */

/**
 *
 * Handles import of txt files with exercises to make it easier for the
 * user to create assignments. The syntax of the txt file is specified on the import main page
 *
 * @author: Christa Deiwiks <cdeiwiks@uos.de>
 */

function vips_import ($subaction)
{
    global $g_outlet;

    if ($g_outlet->status == 'lecturer' || $g_outlet->status == 'tutor') {
        if ($subaction == 'import') {
            import();
        } else {
            main();
        }
    } else {
        StudIPTemplateEngine::showErrorMessage(_('Sie haben keine Rechte, diesen Bereich zu betreten.'));
    }
}

/**
 *
 * Displays import main page
 *
 */

function main() {
        global $g_outlet, $vipsTemplateFactory;

        $message = vips_get_string_array_param('message');
        $t_main = $vipsTemplateFactory->open('import_main');

        $t_main->set_attributes(array(
                'picturepath' => $g_outlet->picturepath,
                'status'      => $g_outlet->status,
                'message'     => $message));

        echo $t_main->render();
}

/**
 *
 * Import function.
 *
 */

function import() {
        global $vips_db,
                   $g_outlet,
                   $g_assignment_table,
                   $g_inAssignment_table;

        $exercise_position      = 1;
        $error = false;
        $dauer = 0;

        if (vips_get_submit_param('import_sheet')) {
                $g_assignment_table     = 'vips_uebungsblatt';
                $g_inAssignment_table   = 'vips_inUebungsblatt';
                $dauer = 10080;                 //default:one week;
        }
        else {  //import_exam
                $g_assignment_table     = 'vips_klausur';
                $g_inAssignment_table   = 'vips_inKlausur';
                $dauer = 120;                           //default:2hrs;
        }

        if (is_uploaded_file($_FILES['userfile']['tmp_name']))
        {
            $my_input = file($_FILES['userfile']['tmp_name']);
        }
        else
        {
            if ($_FILES['userfile']['name'] == '') {
                $message['message'] = 'Sie mssen eine Datei zum Importieren auswhlen.';
            } else {
                $message['message'] = 'Es trat ein Fehler beim Hochladen der Datei auf.';
            }
            $message['mode'] = 'error';
            $error = true;
        }

        $j = 0;

        #########################
        #  get rid of comments  #
        #########################

        if (!$error) {
                for ($i = 0; $i < count($my_input); $i++)
                {
                        if (!preg_match('/^#/', $my_input[$i]))
                        {
                                $my_input2[$j] = $my_input[$i];
                                $j++;
                        }
                }
                $my_input = $my_input2;

                #####################
                # insert assignment #
                #####################

                $assignment_name = basename($_FILES['userfile']['name'], '.txt');

                $now_t  = time() + 60 * 60;
                $now    = date("Y-m-d H:00:00", $now_t);        // unix timestamp
                $ende_t = $now_t + $dauer * 60;                 // + duration
                $ende   = date("Y-m-d H:00:00", $ende_t);

                // find out max Position
                $result = $vips_db->executeQuery(
                        "SELECT MAX(Position) AS pos FROM $g_assignment_table".
                        " WHERE Kurs = '$g_outlet->courseID'");
                $result->next();
                $position = $result->getInt('pos') + 1;

                $vips_db->executeUpdate("INSERT INTO $g_assignment_table SET
                                         Name     = '$assignment_name',
                                         Beginn   = '$now',
                                         Ende     = '$ende',
                                         Dauer    = '$dauer',
                                         Position = '$position',
                                         Kurs     = '$g_outlet->courseID'");

                $vips_idgen = $vips_db->getIdGenerator();
                $assignment_id = $vips_idgen->getId();



        ###################################################
        #                                                 #
        #   P A R S I N G   T H E   W H O L E   F I L E   #
        #                                                 #
        ###################################################

                $i = 0;

                while ($i < count($my_input))
                {
                        $exercise_name          = '';
                        $exercise_type          = '';
                        $exercise_question      = '';
                        $exercise_hint          = '';
                        $exercise_pic_url       = '';
                        $answer_distance        = 0;

                        $exercise_max_points = 1;

                        $answer_counters = array();
                        $answer_counter = 0;
                        $group_counter = 0;

                        #########################
                        #     exercise name     #
                        #########################

                        if (preg_match('/^Name:/', $my_input[$i])) {
                                list($m_split, $m_name) = split(":", $my_input[$i], 2);
                                $i++;
                        } else {
                                $m_name = $assignment_name.' Aufgabe '.$exercise_position;
                        }
                        $exercise_name = trim($m_name);

                        #########################
                        #     exercise type     #
                        #########################

                        if (preg_match("/^SCO-Frage:/",$my_input[$i])) {
                                $exercise_type = "sco_exercise";
                        }
                        elseif (preg_match("/^SC-Frage:/",$my_input[$i]) ){
                                $exercise_type = "sc_exercise";
                        }
                        elseif (preg_match("/^JN-Frage:/",$my_input[$i]) ){
                                $exercise_type = "yn_exercise";
                        }
                        elseif (preg_match("/^MC-Frage:/",$my_input[$i])) {
                                $exercise_type = "mc_exercise";
                        }
                        elseif (preg_match("/^MCO-Frage:/",$my_input[$i]) ){
                                $exercise_type = "mco_exercise";
                        }
                        elseif (preg_match("/^Frage:/",$my_input[$i]) ){
                                $exercise_type = "lt_exercise";
                        }
                        elseif (preg_match("/^PL-Frage:/",$my_input[$i])) {
                                $exercise_type = "pl_exercise";
                        }
                        elseif (preg_match("/^OC-Frage:/",$my_input[$i]) ){
                                $exercise_type = "oc_exercise";
                        }
                        elseif (preg_match("/^OCT-Frage:/",$my_input[$i]) ){
                                $exercise_type = "oct_exercise";
                        }
                        elseif (preg_match("/^Offene Frage:/",$my_input[$i]) ){
                                $exercise_type = "tb_exercise";
                        }
                        elseif (preg_match('/^(ZU|RH)-Frage:/', $my_input[$i]))  {
                            $exercise_type = 'rh_exercise';
                        }
                        else if (preg_match('/^L\'text:/', $my_input[$i]))  {
                            $exercise_type = 'cloze_exercise';
                        }
                        else {
                            // skip empty line or unknown tag
                            $i++;
                            continue;
                        }

                        require_once("exercises/$exercise_type.php");

                        #########################
                        #   exercise question   #
                        #########################

                        list($m_typ, $m_frage) = split(":", $my_input[$i],2);
                        $i++;
                        while (!preg_match("/^Tipp:/", $my_input[$i]) &&
                               !preg_match("/^Grafik:/", $my_input[$i]) &&
                               !preg_match("/^Auswahl:/", $my_input[$i]) &&
                               !preg_match("/^Abgleich:/", $my_input[$i]) &&
                               !preg_match("/^[+~]?Antwort:/", $my_input[$i]) &&
                               !preg_match("/^Name:/", $my_input[$i]) &&
                               !preg_match('/^Vorgabe:/', $my_input[$i]) &&
                               !preg_match('/^\\\\L\'text/', $my_input[$i]) &&
                               $i < count($my_input))
                        {
                                $m_frage .= $my_input[$i];
                                $i++;
                        }
                        $exercise_question = trim($m_frage);

                        ##########
                        #  Tipp  #
                        ##########

                        if (preg_match('/^Tipp:/', $my_input[$i])) {
                                list($m_tag, $m_tipp) = split(':', $my_input[$i], 2);
                                $i++;
                                while (!preg_match('/^\\\\Tipp/', $my_input[$i]) && $i < count($my_input))
                                {
                                        $m_tipp .= $my_input[$i];
                                        $i++;
                                }
                                $exercise_hint = trim($m_tipp);
                                $i++;
                        }

                        #########################
                        #   exercise pic url    #
                        #########################

                        if (preg_match('/^Grafik:/', $my_input[$i])) {
                                list($m_typ, $m_grafik) = split(":", $my_input[$i],2);
                                $exercise_pic_url = $m_grafik;
                                $i++;
                        }

                        #######################
                        #   Antwortauswahl    #
                        #######################

                        $label_yes = _('Ja');
                        $label_no  = _('Nein');

                        if (preg_match('/^Auswahl:/', $my_input[$i])) {
                                list($m_typ, $m_auswahl) = split(":", $my_input[$i],2);
                                list($label_yes, $label_no) = split('/', $m_auswahl);
                                $label_yes = trim($label_yes);
                                $label_no = trim($label_no);
                                $i++;
                        }

                        #######################
                        #   Antwortabgleich   #
                        #######################

                        if (preg_match('/^Abgleich:/', $my_input[$i])) {
                                list($m_typ, $m_abgleich) = split(":", $my_input[$i],2);
                                $m_abgleich = trim($m_abgleich);

                                if ($m_abgleich == 'Levenshtein') {
                                    $answer_distance = 1;
                                } else if ($m_abgleich == 'Soundex') {
                                    $answer_distance = 2;
                                }
                                $i++;
                        }

        /*****************  Single Choice oder J/N-Frage *******************/
                        if ($exercise_type == 'sc_exercise' ||
                            $exercise_type == 'sco_exercise' ||
                            $exercise_type == 'yn_exercise')
                        {
                                while (preg_match('/^\+?Antwort:|^--/', $my_input[$i]))
                                {
                                        if (preg_match('/^\+Antwort:/', $my_input[$i])) {
                                                $_POST["correct_$group_counter"] = "$answer_counter";
                                        } else if (preg_match('/^--/', $my_input[$i])) {
                                                $i++;
                                                $group_counter++;
                                                $answer_counters[] = $answer_counter;
                                                $answer_counter = 0;
                                                continue;
                                        }

                                        list($m_tag, $m_antwort) = split(":",  $my_input[$i],2);
                                        $i++;
                                        while (!preg_match("/^\+?Antwort:|^--/",$my_input[$i]) &&
                                               !preg_match("/^Name:/",$my_input[$i]) &&
                                               !preg_match("/Frage:/",$my_input[$i]) &&
                                               $i < count($my_input))
                                        {
                                                $m_antwort .= $my_input[$i];
                                                $i++;
                                        }

                                        if ($exercise_type == 'yn_exercise') {
                                                if ($answer_counter == 0) {
                                                        $label_yes = $m_antwort;
                                                } else {
                                                        $label_no = $m_antwort;
                                                }
                                        } else {
                                                $key = 'answer_'.$group_counter.'_'.$answer_counter;
                                                $_POST[$key] = $m_antwort;
                                        }
                                        $answer_counter++;
                                }
                                $answer_counters[] = $answer_counter;
                                $answer_counter = implode(',', $answer_counters);
                        }

        /***************** MC, MCO, LT ****************/
                        elseif ($exercise_type == 'mc_exercise' ||
                                $exercise_type == 'mco_exercise' ||
                                $exercise_type == 'lt_exercise')
                        {
                                while (preg_match('/^[+~]?Antwort:/',$my_input[$i]))
                                {
                                        if (preg_match('/^\+Antwort:/',$my_input[$i])) {
                                                $_POST["correct_$answer_counter"] = '1';
                                        } else if (preg_match('/^~Antwort:/',$my_input[$i])) {
                                                $_POST["correct_$answer_counter"] = '0.5';
                                        } else {
                                                $_POST["correct_$answer_counter"] = '0';
                                        }

                                        list($m_tag, $m_antwort) = split(":",  $my_input[$i],2);
                                        $i++;
                                        while (!preg_match("/^[+~]?Antwort:/",$my_input[$i]) &&
                                               !preg_match("/^Name:/",$my_input[$i]) &&
                                               !preg_match("/Frage:/",$my_input[$i]) &&
                                               $i < count($my_input))
                                        {
                                                $m_antwort .= $my_input[$i];
                                                $i++;
                                        }

                                        $_POST["answer_$answer_counter"] =  trim($m_antwort);
                                        $answer_counter++;
                                }

                                if ($exercise_type != 'lt_exercise') {
                                    $exercise_max_points = $answer_counter;
                                }
                        }


        /***************** PL, OC, OCT, Offene Frage ************************/
                        elseif ($exercise_type == 'pl_exercise' ||
                                $exercise_type == 'oc_exercise' ||
                                $exercise_type == 'oct_exercise' ||
                                $exercise_type == 'tb_exercise')
                        {
                                while (preg_match('/^(Vorgabe|Antwort):/', $my_input[$i])) {
                                        list($m_typ, $m_antwort) = split(":",  $my_input[$i], 2);
                                        $i++;

                                        while (!preg_match('/^\\\\(Vorgabe|Antwort)/', $my_input[$i]) &&
                                               $i < count($my_input))
                                        {
                                                $m_antwort .= $my_input[$i];
                                                $i++;
                                        }

                                        if ($m_typ == 'Vorgabe') {
                                            $_POST["answer_default"] = trim($m_antwort);
                                        } else {
                                            $_POST["answer_0"] = trim($m_antwort);
                                        }
                                        $i++;
                                }
                        }


            /********************* Zuordnung *********************/
            else if ($exercise_type == 'rh_exercise')
            {
                $hits = array();

                while (preg_match('/^(Vorgabe:)(.*)/', $my_input[$i], $hits))
                {
                    // $hits == array(0 => whole line, 1 => "Vorgabe:", 2 => rest) //
                    $_POST["default_$answer_counter"] = trim($hits[2]);
                    $i++;

                    preg_match('/^(Antwort:)(.*)/', $my_input[$i], $hits);
                    $_POST["answer_$answer_counter"] = trim($hits[2]);
                    $_POST["correct_$answer_counter"] = "$answer_counter";

                    $i++;
                    $answer_counter++;  // determines $_POST['number_of_answers'] below
                }
            }



                        ################################
                        #  write exercise to database  #
                        ################################

                        if ($exercise_type) {
                                $_POST['exercise_type']     = $exercise_type;
                                $_POST['exercise_question'] = $exercise_question;
                                $_POST['exercise_name']     = $exercise_name;
                                $_POST['exercise_hint']     = $exercise_hint;
                                $_POST['exercise_pic_url']  = $exercise_pic_url;
                                $_POST['number_of_answers'] = $answer_counter;
                                $_POST['answer_distance']   = $answer_distance;
                                $_POST['label_yes']         = $label_yes;
                                $_POST['label_no']          = $label_no;

                                $m_Oclass = new $exercise_type();
                                $m_exerciseXML =  $m_Oclass->getExerciseConstruct();
                                // use addslashes for XML: special characters (as "'") might occur;
                                // wouldn't work in a mysql query

                                // mix answers in mapping exercise
                                if ($exercise_type == 'rh_exercise')
                                {
                                    $m_exerciseXML = $m_Oclass->mix_answers($m_exerciseXML);
                                }

                                if ($exercise_type == 'cloze_exercise')
                                {
                                    $exercise_max_points = $m_Oclass->getNumberOfAnswers();
                                }

                                /*
                                echo "<br><b>Importiert: ";
                                echo "Aufgabe ".$exercise_position." - \"".$exercise_name."\"</b>";
                                echo "<br>Typ: ".$exercise_type;
                                echo "<br>Frage: ".$exercise_question;
                                */

                                $query = "INSERT INTO vips_aufgabe
                                          SET Name    = '".addslashes($exercise_name)."',
                                              Aufgabe = '".addslashes($m_exerciseXML)."',
                                              URI     = '$exercise_type',
                                              Userid  = '$g_outlet->userID'";
                                $vips_db->executeUpdate($query);

                                $vips_idgen = $vips_db->getIdGenerator();
                                $exercise_id = $vips_idgen->getId();

                                $query = "INSERT INTO $g_inAssignment_table
                                          SET vips_aufgabe        = '$exercise_id',
                                              $g_assignment_table = '$assignment_id',
                                              Punkte              = '$exercise_max_points',
                                              Position            = '$exercise_position',
                                              Bewertet            = 't'";
                                $vips_db->executeUpdate($query);

                                $exercise_position++;

                                // unset all POST parameters
                                foreach (array_keys($_POST) as $key) {
                                    unset($_POST[$key]);
                                }
                        }
                } // end: for all the input

                $message['message'] = 'Die Aufgaben wurden importiert. Bitte bearbeiten Sie jetzt die Grunddaten des bungsblattes/der Klausur!';
                $message['mode']    = 'msg';
        }       //end if no error

        $_POST['message']   = $message;

        main();
} // end function import()

?>
